#ifndef HARKIO_HARKCONFIG_XMLHELPER_HPP_
#define HARKIO_HARKCONFIG_XMLHELPER_HPP_

#include <map>
#include <glibmm/ustring.h>
#include <libxml++/parsers/textreader.h>

namespace harkio2
{

class AnalyzeError;

namespace xmlhelper
{

/** @brief Returns true if the specified map has the specified key.
 * @param m A map.
 * @param k A key.
 * @return true if the specified map has the specified key.
 */
template <typename Value>
bool Contains(const std::map<Glib::ustring, Value>& m, const Glib::ustring& k);

/** @brief Returns the line number at which the reader is.
 * @param reader A TextReader.
 * @return The line number at which the reader is.
 */
int GetLine(const xmlpp::TextReader& reader);

/** @brief Returns true if the current node is start tag which name is 'name'.
 * @param reader A TextReader.
 * @param name A tag name.
 * @return true if the current node is start tag which name is 'name'.
 */
bool IsElement(const xmlpp::TextReader& reader, const Glib::ustring& name);

/** @brief Returns true if the current node is start tag which name is 'name'.
 * @param reader A TextReader.
 * @param name A tag name.
 * @return true if the current node is start tag which name is 'name'.
 */
bool IsElement(const xmlpp::TextReader& reader, const char* name);

/** @brief Returns true if the current node is end tag which name is 'name'.
 * @param reader A TextReader.
 * @param name A tag name.
 * @return true if the current node is end tag which name is 'name'.
 */
bool IsEndElement(const xmlpp::TextReader& reader, const Glib::ustring& name);

/** @brief Returns true if the current node is end tag which name is 'name'.
 * @param reader A TextReader.
 * @param name A tag name.
 * @return true if the current node is end tag which name is 'name'.
 */
bool IsEndElement(const xmlpp::TextReader& reader, const char* name);

/** @brief Moves to next start tag and returns true if the movement succeed.
 * This function finds next start tag.
 * Finding starts from the position where 'reader' currently is.
 * If the next start tag is found, 'reader' will be on the found tag.
 * Otherwise 'reader' will be on the end of file.
 * @param reader[in,out] A textreader
 * @return true if the movement succeed.
 */
bool MoveToNextElement(xmlpp::TextReader& reader);

/** @brief Moves to next start tag whose name is 'name'.
 * This function finds next start tag whose name is 'name'.
 * Finding starts from the position where 'reader' currently is.
 * If the next 'name' tag is found, 'reader' will be on the found tag.
 * Otherwise 'reader' will be on the end of file.
 * @param reader[in,out] A textreader
 * @param name The name to find.
 * @return true if the movement succeed.
 */
bool FindElement(xmlpp::TextReader& reader, const Glib::ustring& name);

/* implement */
template <typename Value>
bool Contains(const std::map<Glib::ustring, Value>& m, const Glib::ustring& k)
{
    return m.find(k) != m.end();
}

} // xmlhelper

} // harkio

#endif // HARKIO_HARKCONFIG_XMLHELPER_HPP_

